"""Functions for formatting output."""

import subprocess
from typing import List


def format_subprocess(args: List[str]) -> str:
    """Formats the output to remove newlines and decode to utf-8.

    Args:
        args: A list of arguments to run

    Returns:
        str: The formatted output of the result

    """
    return subprocess.check_output(args).decode("utf-8").strip()



"""Functions for cleaning up after completion, such as resetting files to their original state."""

import subprocess
import tempfile

import click

from seaport.checks import user_path


def clean(original_text: str, location: str, port_name: str) -> None:
    """Returns the user's local portfile repo to the original state.

    Args:
        original_text: What the contents of the portfile originally was
        location: Where the portfile is located
        port_name: The name of the portfile

    """
    click.secho("🧽 Cleanup", fg="cyan")
    # Change contents of local portfile back to original
    tmp_original = tempfile.NamedTemporaryFile(mode="w")
    tmp_original.write(original_text)
    tmp_original.seek(0)
    subprocess.run(
        [f"{user_path()}/sudo", "cp", tmp_original.name, location], check=True
    )
    tmp_original.close()

    subprocess.run(
        [f"{user_path()}/sudo", f"{user_path(True)}/port", "clean", "--all", port_name],
        check=True,
    )



"""Functions related to determining the current and new checksums."""

import shutil
import sys
import tempfile
import urllib.request
from pathlib import Path
from typing import Tuple

import click

from seaport.checks import user_path
from seaport.format import format_subprocess


def new_checksums(website: str) -> Tuple[str, str, str]:
    """Generate checksums of file downloaded from website.

    Args:
        website: Where to download the new file from

    Returns:
        Tuple[str, str, str]: A tuple of strings representing the new checksums
    """
    download_dir = tempfile.TemporaryDirectory()
    download_location = f"{download_dir.name}/download"

    # Download the file from `url` and save it locally under `file_name`:
    # Originally urllib.request.urlretrieve(website, download_location), but this is depreciated
    # Credit https://stackoverflow.com/a/7244263
    with urllib.request.urlopen(website) as response, open(
        download_location, "wb"
    ) as out_file:
        shutil.copyfileobj(response, out_file)

    # sha256 flag added even though it's the default
    # Otherwise sometimes doesn't return sha256
    sha256 = format_subprocess(
        [f"{user_path()}/openssl", "dgst", "-sha256", download_location]
    ).split(" ")[-1]
    rmd160 = format_subprocess(
        [f"{user_path()}/openssl", "dgst", "-rmd160", download_location]
    ).split(" ")[-1]
    size = str(Path(download_location).stat().st_size)

    download_dir.cleanup()

    return sha256, rmd160, size


def current_checksums(port: str, current: str, new: str) -> Tuple[str, str, str, str]:
    """Returns outdated checksums and the website to download the new files from.

    Args:
        port: Name of the port
        current: The current and outdated version number
        new: The new version number

    Returns:
        Tuple[str, str, str, str]: A tuple of strings representing the new website and checksums
    """
    distfiles = (
        format_subprocess([f"{user_path(True)}/port", "distfiles", port])
        .replace("\n ", "")
        .split(" ")
    )

    # There's no output if it's the "skeleton" head port
    try:
        old_website = [s for s in distfiles if "http" in s][0]
    except IndexError:
        # Tries to determine the subport
        # This is since the distfiles cmd only works for subports
        port_info = format_subprocess([f"{user_path(True)}/port", "info", port])
        if "Sub-ports" not in port_info:
            click.secho("Cannot determine distfiles", fg="red")
            sys.exit(1)
        # Takes the last subport of the list
        subport = " ".join(
            [s for s in port_info.splitlines() if "Sub-ports" in s]
        ).split(" ")[-1]
        # Repeat the process with the subport
        return current_checksums(subport, current, new)

    new_website = old_website.replace(current, new)

    website_index = distfiles.index(old_website)

    # This won't work for older portfiles (e.g. if they used md5 and sha1 for example)
    size = distfiles[website_index - 1]
    sha256 = distfiles[website_index - 2][:-5]  # Remove size:
    rmd160 = distfiles[website_index - 3][:-7]  # Remove sha256:

    return new_website, size, sha256, rmd160



"""The main CLI function, which the user runs."""

import os
import subprocess
import sys
import tempfile
from typing import Optional

import click

from seaport.additional import perform_lint
from seaport.checks import preliminary_checks, user_path
from seaport.clean import clean
from seaport.commands.autocomplete.autocomplete import get_names
from seaport.format import format_subprocess
from seaport.portfile.checksums import current_checksums, new_checksums
from seaport.portfile.portfile_numbers import new_version, undo_revision


@click.command()
@click.argument("name", type=str, autocompletion=get_names)
# Some versions could be v1.2.0-post for example
@click.option("--bump", help="The new version number", type=str)
@click.option("--test/--no-test", default=False, help="Runs port test")
@click.option("--lint/--no-lint", default=False, help="Runs port lint --nitpick")
@click.option(
    "--install/--no-install",
    default=False,
    help="Installs the port and allows testing of basic functionality",
)
# Some parameters are not used
# They are only here since the pr function sends them
def clip(
    name: str,
    bump: str,
    test: bool,
    lint: bool,
    install: bool,
    location: Optional[str] = None,
    new: Optional[bool] = None,
) -> None:
    """Bumps the version number and checksum of NAME, and copies the result to your clipboard."""
    # Tasks that require sudo
    sudo = test or lint or install

    preliminary_checks(name, location)

    current_version = format_subprocess(
        [f"{user_path(True)}/port", "info", "--version", name]
    ).split(" ")[1]

    # Determine new version
    bump = new_version(name, bump, current_version)

    click.secho(f"👍 New version is {bump}", fg="green")

    # Allows pr function to get the version number
    os.environ["BUMP"] = bump

    # Where to download the new file + old checksums
    new_website, old_size, old_sha256, old_rmd160 = current_checksums(
        name, current_version, bump
    )

    click.secho(f"🔻 Downloading from {new_website}", fg="cyan")
    new_sha256, new_rmd160, new_size = new_checksums(new_website)

    click.secho("🔎 Checksums:", fg="cyan")
    click.echo(f"Old rmd160: {old_rmd160}")
    click.echo(f"New rmd160: {new_rmd160}")
    click.echo(f"Old sha256: {old_sha256}")
    click.echo(f"New sha256: {new_sha256}")
    click.echo(f"Old size: {old_size}")
    click.echo(f"New size: {new_size}")

    # Add the new checksums, and take a backup of the original
    file_location = (
        subprocess.check_output([f"{user_path(True)}/port", "file", name])
        .decode("utf-8")
        .strip()
    )

    with click.open_file(file_location) as file:
        # Backup of the original contents
        original = file.read()

    # Bump revision numbers to 0
    new_contents = undo_revision(original)

    # Replace first instances only
    new_contents = new_contents.replace(current_version, bump, 1)
    new_contents = new_contents.replace(old_sha256, new_sha256, 1)
    new_contents = new_contents.replace(old_rmd160, new_rmd160, 1)
    new_contents = new_contents.replace(old_size, new_size, 1)

    if sudo:

        # Temporary files created to get around sudo write problem
        tmp_version = tempfile.NamedTemporaryFile(mode="w")
        tmp_version.write(new_contents)
        tmp_version.seek(0)

        click.secho("💾 Editing local portfile repo, sudo required", fg="cyan")
        click.echo("Changes will be reverted after completion")
        subprocess.run(
            [f"{user_path()}/sudo", "cp", tmp_version.name, file_location], check=True
        )

        if test:
            click.secho(f"🧪 Testing {name}", fg="cyan")
            subprocess.run(
                [f"{user_path()}/sudo", f"{user_path(True)}/port", "test", name],
                check=True,
            )
        if lint:
            # If the lint is not successful
            if not perform_lint(name):
                clean(original, file_location, name)
                sys.exit(1)

        if install:
            click.secho(f"🏗️ Installing {name}", fg="cyan")
            subprocess.run(
                [
                    f"{user_path()}/sudo",
                    f"{user_path(True)}/port",
                    "-vst",
                    "install",
                    name,
                ],
                check=True,
            )
            click.secho(
                "Paused to allow user to test basic functionality in a different terminal",
                fg="cyan",
            )
            click.pause("Press any key to continue ")
            click.secho(f"🗑 Uninstalling {name}", fg="cyan")
            subprocess.run(
                [f"{user_path()}/sudo", f"{user_path(True)}/port", "uninstall", name],
                check=True,
            )

        clean(original, file_location, name)

        subprocess.run(
            f"{user_path()}/pbcopy",
            universal_newlines=True,
            input=new_contents,
            check=True,
        )

    click.secho(
        "📋 The contents of the portfile have been copied to your clipboard!",
        fg="cyan",
    )



"""Functions related to bumping revision and version numbers of a portfile."""

import re
import sys

import click

from seaport.checks import user_path
from seaport.format import format_subprocess


def undo_revision(text: str) -> str:
    """Make version numbers 0.

    Args:
        text: The text of the portfile

    Returns:
        str: The text with version numbers decremented to 0

    """
    click.secho("⏪️ Changing revision numbers", fg="cyan")

    # Counts no. of revision numbers greater than 0
    # Assumes revision number doesn't exceed 9
    need_changed = len(re.findall(r"revision\s*[1-9]", text))
    total = len(re.findall(r"revision\s*", text))

    # If there are no revisions greater than 1, do nothing
    if need_changed == 0:
        click.echo("No changes necessary")
        return text
    # If there are multiple revision numbers, we don't know which one to change
    # If all of the revision numbers are 0, this is accounted for in the last if statement
    if total > 1:
        click.secho(
            "Multiple revision numbers found. Unsure which to reduce to 0", fg="red"
        )
        sys.exit(1)

    # Implies need_changed == 1

    # Takes the original revision as a string
    # We know original_revision will not be of type none (hence mypy ignore)
    # This is ince need_changed is equal to 1
    original_revision = re.search(r"revision\s*[1-9]", text).group(0)  # type: ignore
    # Replaces the number with 0
    new_revision = original_revision[:-1] + "0"
    click.echo("Revision number changed")
    return text.replace(original_revision, new_revision)


def new_version(port: str, stated: str, current: str) -> str:
    """Determines livecheck version, and sees whether already up-to-date.

    Args:
        port: The name of the port
        stated: The user's new version via --bump
        current: The current version of the port

    Returns:
        str: The version number following checks

    """
    # Determines new version number if none manually specified
    if not stated:
        # Take the last word of port livecheck, and then remove the bracket
        stated = format_subprocess(
            [f"{user_path(True)}/port", "livecheck", port]
        ).split(" ")[-1][:-1]

        # version == "" if livecheck doesn't output anything
        # current_version used in output since version = ""
        if stated == "":
            click.secho(
                f"{port}'s either already up-to-date ({current}) or there's no livecheck available",
                fg="red",
            )
            click.secho("Please manually specify the version using --bump", fg="red")
            sys.exit(1)

    if stated == current:
        click.secho(f"{port} is already up-to-date ({current})", fg="red")
        sys.exit(1)

    # Credit to @herbygillot
    # See https://github.com/macports/macports-ports/pull/9589#issuecomment-753309298
    # alpha/beta/rc version detected on a port that isn't -devel
    devel_versions = ["alpha", "beta", "rc", "devel", "dev", "unstable"]
    if "-devel" not in port and any(item in stated for item in devel_versions):
        if not click.confirm(
            f"{port} is not a devel port, but the new version ({stated}) is a devel build. Do you wish to continue?"
        ):
            click.echo("You can specify a different version using --bump")
            sys.exit(1)

    return stated



"""Defensive programming functions."""

import subprocess
import sys
from shutil import which
from typing import Optional

import click

from seaport.format import format_subprocess


def user_path(port: bool = False, third_party: bool = False) -> str:
    """Determines the path to prevent starting a process with a partial executable path.

    Args:
        port: Whether to output the path of the port cmd
        third_party: Whether the dependency isn't installed by default

    Returns:
        A string representing the path

    Path to run system commands (e.g. git)
    See https://bandit.readthedocs.io/en/latest/plugins/b607_start_process_with_partial_path.html
    """
    # no forward slash at end for Bandit B607

    port_path = format_subprocess(["/usr/bin/which", "port"])
    port_prefix = port_path.split("bin")[0] + "bin"

    if port:
        return port_prefix

    if third_party:
        # Cannot use port_path in case not installed by MacPorts (e.g. Homebrew)
        seaport_path = format_subprocess(["/usr/bin/which", "seaport"])

        if "Python" not in seaport_path and "virtualenvs" not in seaport_path:
            # Can't run system commands from python path
            return seaport_path.split("bin")[0] + "bin"

        # Default to standard MacPorts prefix
        return port_prefix

    return "/usr/bin"


def cmd_check(name: str) -> bool:
    """Checks whether a command is installed.

    Args:
        name: Name of the port

    Returns:
        bool: Whether the command is active
    """
    # Credit to https://stackoverflow.com/a/34177358/10763533
    return which(name) is not None


def exists(name: str) -> None:
    """Checks whether the port exists.

    Args:
        name: Name of the port
    """
    # Hide output
    if subprocess.call(
        [f"{user_path(True)}/port", "info", name],
        stdout=subprocess.DEVNULL,
        stderr=subprocess.STDOUT,
    ):
        click.secho(f"❌ {name} is not a port", fg="red")
        sys.exit(1)


def preliminary_checks(port: str, pull_request: Optional[str]) -> None:
    """Checks to run before carrying out updating process.

    Args:
        port: Name of the port
        pull_request: Path of where to clone the macports-ports repo
    """
    if not cmd_check("port"):
        click.secho("❌ MacPorts not installed", fg="red")
        click.echo("It can be installed from https://www.macports.org/")
        sys.exit(1)
    elif pull_request and not cmd_check("gh"):
        # gh only required if sending pr
        click.secho("❌ Github CLI not installed", fg="red")
        if not click.confirm("Do you want to install this via MacPorts?"):
            sys.exit(1)
        subprocess.run(
            [f"{user_path()}/sudo", f"{user_path(True)}/port", "install", "gh"],
            check=True,
        )
    exists(port)



"""Command to send a PR after updating portfile."""

import os
import subprocess
import sys
import tempfile
from shutil import copyfile
from typing import Any

import click

from seaport.checks import user_path
from seaport.commands.autocomplete.autocomplete import get_names
from seaport.commands.clipboard import clip
from seaport.format import format_subprocess


@click.command()
@click.argument("name", type=str, autocompletion=get_names)
@click.argument(
    "location",
    type=click.Path(exists=True, dir_okay=True, writable=True),
)
# Some versions could be v1.2.0-post for example
@click.option("--bump", help="The new version number", type=str)
@click.option("--test/--no-test", default=False, help="Runs port test")
@click.option("--lint/--no-lint", default=False, help="Runs port lint --nitpick")
@click.option(
    "--install/--no-install",
    default=False,
    help="Installs the port and allows testing of basic functionality",
)
@click.option("--new", is_flag=True, help="Send a PR from the local portfile repo")
@click.pass_context
def pr(
    ctx: Any,
    name: str,
    bump: str,  # bump is used as part of ctx.forward
    location: str,
    test: bool,
    lint: bool,
    install: bool,
    new: bool,
) -> None:
    """Bumps the version number and checksum of NAME, and sends a PR to update it."""
    # Invoke the clipboard cmd
    # That's the command that determines the new contents
    ctx.forward(clip)

    # Determine the output of the clip function from clipboard
    # Newline added since clipboard removes it
    new_contents = format_subprocess(["pbpaste"]) + "\n"

    # Get updated version number from clip
    # Separate var to get mypy to work
    # This is since os.getenv is Optional[str]
    env_variable = os.getenv("BUMP")

    if env_variable is None:
        click.secho(
            "Cannot determine version number from env variable `bump`", fg="red"
        )
        sys.exit(1)

    bump = env_variable

    # Write the new portfile contents to a tempfile
    tmp_version = tempfile.NamedTemporaryFile(mode="w")
    tmp_version.write(new_contents)
    tmp_version.seek(0)

    # Category determined so as to know where to put the portfile
    # e.g. macports-ports/category/name/Portfile
    category_list = format_subprocess(
        [f"{user_path(True)}/port", "info", "--category", name]
    ).split(" ")

    # Remove comma, and only take the first category
    if len(category_list) > 2:
        category = category_list[1][:-1]
    else:
        category = category_list[1]

    click.secho("🚀 Cloning macports/macports-ports", fg="cyan")
    os.chdir(location)
    # check false if macports-ports already exists (error 127)
    subprocess.run(
        [
            f"{user_path(False, True)}/gh",
            "repo",
            "fork",
            "macports/macports-ports",
            "--clone=true",
            "--remote=true",
        ],
        check=False,
    )

    # Update origin
    os.chdir(f"{location}/macports-ports")
    subprocess.run([f"{user_path()}/git", "fetch", "upstream"], check=True)
    subprocess.run([f"{user_path()}/git", "merge", "upstream/master"], check=True)
    subprocess.run([f"{user_path()}/git", "push"], check=True)

    # Different titles depending on whether updating
    # or adding new file
    commit_title = f"{name}: new port" if new else f"{name}: update to {bump}"

    subprocess.run(
        [f"{user_path()}/git", "checkout", "-b", f"seaport-{name}-{bump}"],
        check=True,
    )

    # Remove backslash from user macports repo location
    # This allows copyfile to work regardless of whether there's
    # a backslash or not
    location = location.rstrip("/")

    if new:
        # Have to create directories for new portfile
        subprocess.run(
            ["/bin/mkdir", "-p", f"{location}/macports-ports/{category}/{name}"],
            check=True,
        )
        # copyfile not used since cp creates the file automatically
        # IF copyfile was used, touch would also be required
        subprocess.run(
            [
                "/bin/cp",
                tmp_version.name,
                f"{location}/macports-ports/{category}/{name}/Portfile",
            ],
            check=True,
        )
    else:
        copyfile(
            tmp_version.name,
            f"{location}/macports-ports/{category}/{name}/Portfile",
        )

    subprocess.run(
        [f"{user_path()}/git", "add", f"{category}/{name}/Portfile"], check=True
    )
    subprocess.run(
        [f"{user_path()}/git", "commit", "-m", commit_title],
        check=True,
    )
    # Automatically choose to send PR to remote
    # Change to remote.origin.gh-resolved to send to user's fork
    subprocess.run(
        [f"{user_path()}/git", "config", "remote.upstream.gh-resolved", "base"],
        check=True,
    )

    # PR variables
    mac_version = format_subprocess([f"{user_path()}/sw_vers", "-productVersion"])
    xcode_version = format_subprocess(
        [f"{user_path()}/xcodebuild", "-version"]
    ).replace("\nBuild version", "")

    if click.confirm("Does everything look good before sending PR?"):
        subprocess.run(
            [
                f"{user_path()}/git",
                "push",
                "--set-upstream",
                "origin",
                f"seaport-{name}-{bump}",
            ],
            check=True,
        )
        subprocess.run(
            [
                f"{user_path(False, True)}/gh",
                "pr",
                "create",
                "--title",
                commit_title,
                "--body",
                f"""#### Description

Created with [seaport](https://github.com/harens/seaport)

###### Type(s)

- [ ] bugfix
- [x] enhancement
- [ ] security fix

###### Tested on
macOS {mac_version}
{xcode_version}

###### Verification <!-- (delete not applicable items) -->
Have you

- [x] followed our [Commit Message Guidelines](https://trac.macports.org/wiki/CommitMessages)?
- [x] squashed and [minimized your commits](https://guide.macports.org/#project.github)?
- [ ] checked that there aren't other open [pull requests](https://github.com/macports/macports-ports/pulls) for the same change?
- [ ] referenced existing tickets on [Trac](https://trac.macports.org/wiki/Tickets) with full URL? <!-- Please don't open a new Trac ticket if you are submitting a pull request. -->
- [{"x" if lint else " "}] checked your Portfile with `port lint`?
- [{"x" if test else " "}] tried existing tests with `sudo port test`?
- [{"x" if install else " "}] tried a full install with `sudo port -vst install`?
- [{"x" if install else " "}] tested basic functionality of all binary files?""",
            ],
            check=True,
        )
    # cleanup process
    subprocess.run([f"{user_path()}/git", "checkout", "master"], check=True)
    subprocess.run(
        [f"{user_path()}/git", "branch", "-D", f"seaport-{name}-{bump}"], check=True
    )



"""Additional checks that aren't provided by default.

e.g. Linting
"""

import click

from seaport.checks import user_path
from seaport.format import format_subprocess


def perform_lint(name: str) -> bool:
    """Lints the port and checks output for errors.

    Args:
        name: The name of the port

    Returns:
        bool: Whether the linting was successful or not
    """
    click.secho("🤔 Linting", fg="cyan")
    lint_output = format_subprocess(
        [f"{user_path(True)}/port", "lint", "--nitpick", name]
    )
    click.echo(lint_output)
    output_list = lint_output.split(" ")

    # Finds the no. of errors and warnings
    errors = int(output_list[output_list.index("errors") - 1])
    warnings = int(output_list[output_list.index("warnings") - 1])

    if errors > 1:
        # Fail if there are any errors
        return False
    if warnings > 1:
        # Ask whether the user wishes to continue
        if not click.confirm(
            f"There are {warnings} warnings. Do you wish to continue?"
        ):
            return False
    return True



"""The main cli module (a facade for the other commands)."""

import click

from seaport import __version__
from seaport.commands.clipboard import clip
from seaport.commands.pull_request import pr


@click.group()
@click.version_option(__version__)
def seaport() -> None:
    """Bumps the version number and checksum of a port.

    For more information, please visit https://github.com/harens/seaport
    """
    click.secho("🌊 Starting seaport...", fg="cyan")


seaport.add_command(clip)
seaport.add_command(pr)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['seaport',
 'seaport.commands',
 'seaport.commands.autocomplete',
 'seaport.portfile']

package_data = \
{'': ['*'], 'seaport.commands.autocomplete': ['bash/*', 'fish/*', 'zsh/*']}

install_requires = \
['click>=7.1.2,<8.0.0']

entry_points = \
{'console_scripts': ['seaport = seaport.init:seaport']}

setup_kwargs = {
    'name': 'seaport',
    'version': '0.3.1',
    'description': 'A more mighty port bump',
    'long_description': '# 🌊 seaport\n\n*A more mighty `port bump` for MacPorts!*\n\n| Test Status | [![GitHub Workflow Status](https://img.shields.io/github/workflow/status/harens/seaport/Tests?logo=github&style=flat-square)](https://github.com/harens/seaport/actions?query=workflow%3ATests) [![GitHub Workflow Status](https://img.shields.io/github/workflow/status/harens/seaport/ShellCheck?label=ShellCheck&logo=github%20actions&logoColor=white&style=flat-square)](https://github.com/harens/seaport/actions?query=workflow%3AShellCheck) [![Codecov](https://img.shields.io/codecov/c/github/harens/seaport?logo=codecov&style=flat-square)](https://codecov.io/gh/harens/seaport)  |\n|:--|:--|\n| __Version Info__ | [![PyPI](https://img.shields.io/pypi/v/seaport?logo=pypi&logoColor=white&style=flat-square)](https://pypi.org/project/seaport/) [![GitHub tag (latest by date)](https://img.shields.io/github/v/tag/harens/seaport?logo=github&style=flat-square)](https://github.com/harens/seaport/releases) [![PyPI - Downloads](https://img.shields.io/pypi/dm/seaport?logo=python&logoColor=white&style=flat-square)](https://pypi.org/project/seaport/) |\n| __Code Analysis__ |[![LGTM Grade](https://img.shields.io/lgtm/grade/python/github/harens/seaport?logo=lgtm&style=flat-square)](https://lgtm.com/projects/g/harens/seaport/) [![Code Climate maintainability](https://img.shields.io/codeclimate/maintainability/harens/seaport?logo=code%20climate&style=flat-square)](https://codeclimate.com/github/harens/seaport) [![CodeFactor Grade](https://img.shields.io/codefactor/grade/github/harens/seaport?logo=codefactor&style=flat-square)](https://www.codefactor.io/repository/github/harens/seaport) [![Codacy grade](https://img.shields.io/codacy/grade/8539131738c3433f8057e65aab21de03?logo=codacy&style=flat-square)](https://app.codacy.com/gh/harens/seaport/dashboard?branch=master)|\n\n## ✨ Features\n\n* __Automatically determines new version numbers and checksums__ for MacPorts portfiles.\n* __Copies the changes to your clipboard 📋__, and optionally __sends a PR to update them__.\n* Contains __additional checking functionality__, such as running tests, linting and installing the updated program.\n* __[PEP 561 compatible](https://www.python.org/dev/peps/pep-0561)__, with built in support for type checking.\n\n## 🤖 Example\n\n```\n> seaport clip gping\n🌊 Starting seaport...\n👍 New version is 1.2.0-post\n🔻 Downloading from https://github.com/orf/gping/tarball/v1.2.0-post/gping-1.2.0-post.tar.gz\n🔎 Checksums:\nOld rmd160: 8b274132c8389ec560f213007368c7f521fdf682\nNew rmd160: 4a614e35d4e1e496871ee2b270ba8836f84650c6\nOld sha256: 1879b37f811c09e43d3759ccd97d9c8b432f06c75a27025cfa09404abdeda8f5\nNew sha256: 1008306e8293e7c59125de02e2baa6a17bc1c10de1daba2247bfc789eaf34ff5\nOld size: 853432\nNew size: 853450\n⏪️ Changing revision numbers\nNo changes necessary\n📋 The contents of the portfile have been copied to your clipboard!\n```\n\n## ⬇️ Install\n\nNote that if installing from PyPi or building from source, [MacPorts](https://www.macports.org/) needs to already be installed, and [GitHub CLI](https://cli.github.com/) is required if using the `--pr` flag.\n\n### Homebrew 🍺\n\nBinary bottles are available for x86_64_linux, catalina and big_sur.\n\n```\nbrew install harens/tap/seaport\n```\n\n### PyPi 🐍\n\n```\npip install seaport\n```\n\n## 💻 Usage\n\n```txt\n> seaport --help\nUsage: seaport [OPTIONS] COMMAND [ARGS]...\n\n  Bumps the version number and checksum of a port.\n\n  For more information, please visit https://github.com/harens/seaport\n\nOptions:\n  --version  Show the version and exit.\n  --help     Show this message and exit.\n\nCommands:\n  clip  Bumps the version number and checksum of NAME, and copies the\n        result...\n\n  pr    Bumps the version number and checksum of NAME, and sends a PR to...\n```\n\n### 📋 `seaport clip NAME`\n\nCopies the updated portfile to your clipboard\n\n```txt\n> seaport clip --help\n🌊 Starting seaport...\nUsage: seaport clip [OPTIONS] NAME\n\n  Bumps the version number and checksum of NAME, and copies the result to\n  your clipboard.\n\nOptions:\n  --bump TEXT               The new version number\n  --test / --no-test        Runs port test\n  --lint / --no-lint        Runs port lint --nitpick\n  --install / --no-install  Installs the port and allows testing of basic\n                            functionality\n\n  --help                    Show this message and exit.\n```\n\n#### 📛 `name` (_Required_)\n\nThe name of the port to update.\n\ne.g. `seaport clip gping`\n\n#### 🔻 `--bump TEXT`\n\nManually set the version number to bump it to. This should be in the same format as the output of the livecheck.\n\nBy default, it uses the value outputted from the livecheck.\n\nThis flag can be useful if there\'s no livecheck available or you want to override it.\n\ne.g. `seaport clip gping --bump 1.2.0-post`\n\n#### 🧪 `--test / --no-test` (_default `--no-test`_)\n\nRuns `sudo port test NAME` after updating the portfile.\n\ne.g. `seaport clip py-rich --test`\n\n#### 🤔 `--lint / --no-lint` (_default `--no-lint`_)\n\nRuns `port lint --nitpick NAME` after updating the portfile.\n\ne.g. `seaport clip gping --lint`\n\n#### 🔨 `--install / --no-install` (_default `--no-install`_)\n\nInstalls the port via the updated portfile and allows testing of basic functionality. After this has been completed, the port is uninstalled from the user\'s system.\n\ne.g. `seaport clip gping --install`\n\n### ✉️ `seaport pr NAME`\n\nSends a pull requrest to [macports/macports-ports](https://github.com/macports/macports-ports) with the updated portfile contents.\n\nThe flags in `clip` are also valid for this subcommand.\n\nThe pull request template is automatically filled in depending on what flags the command was run with (e.g. if `--lint` was used, this would be noted in the verification section of the template).\n\n```txt\n> seaport pr --help\n🌊 Starting seaport...\nUsage: seaport pr [OPTIONS] NAME LOCATION\n\n  Bumps the version number and checksum of NAME, and sends a PR to update\n  it.\n\nOptions:\n  --bump TEXT               The new version number\n  --test / --no-test        Runs port test\n  --lint / --no-lint        Runs port lint --nitpick\n  --install / --no-install  Installs the port and allows testing of basic\n                            functionality\n\n  --new                     Send a PR from the local portfile repo\n  --help                    Show this message and exit.\n  ```\n  \n#### ✅ `--new`\n\nThis flag is used if sending a PR for a new portfile from the user\'s [local portfile repo](https://guide.macports.org/chunked/development.local-repositories.html).\n\nTo bypass the version number checks, it is recommended to set a different version number within the file. This is corrected automatically by seaport.\n\n### 🚀 Use of sudo\n\nSudo is only required if `--test`, `--lint` or `--install` are specified, and it will be asked for during runtime. This is since the local portfile repo needs to be modified to be able to run the relevant commands.\n\nAny changes made to the local portfile repo are reverted during the cleanup stage.\n\n## 🔨 Contributing\n\nAny change, big or small, that you think can help improve this action is more than welcome 🎉.\n\nAs well as this, feel free to open an issue with any new suggestions or bug reports. Every contribution is appreciated.\n\n## 📒 Notice of Non-Affiliation and Disclaimer\n\n<img src="https://avatars2.githubusercontent.com/u/4225322?s=280&v=4" align="right"\n     alt="MacPorts Logo" width="150">\n\nThis project is not affiliated, associated, authorized, endorsed by, or in any way officially connected with the MacPorts Project, or any of its subsidiaries or its affiliates. The official MacPorts Project website can be found at <https://www.macports.org>.\n\nThe name MacPorts as well as related names, marks, emblems and images are registered trademarks of their respective owners.\n',
    'author': 'harens',
    'author_email': 'harensdeveloper@gmail.com',
    'maintainer': 'harens',
    'maintainer_email': 'harensdeveloper@gmail.com',
    'url': 'https://github.com/harens/seaport',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Shell completion for port names."""

from typing import Any, List, Tuple, Union

from seaport.checks import user_path
from seaport.format import format_subprocess


def get_names(
    ctx: Any, args: List[str], incomplete: str
) -> List[Union[str, Tuple[str, str]]]:
    """Shell autocompletion for port names.

    Args:
        ctx: The current command context
        args: The list of arguments passed in
        incomplete: The partial word that is being completed

    Returns:
        List[Union[str, Tuple[str, str]]]: The portname and the description
    """
    results = format_subprocess(
        [
            f"{user_path(True)}/port",
            "search",
            "--name",
            "--line",
            "--glob",
            f"{incomplete}*",
        ]
    ).splitlines()
    # Converts to raw string literal to split by backslash
    # See https://stackoverflow.com/a/25047988/10763533
    return [(repr(k).split("\\")[0][1:], repr(k).split("\\")[3][1:-1]) for k in results]
